@@ -1,4 +1,5 @@
 import reversion
+from django.shortcuts import get_object_or_404
 from rest_framework import status
 from rest_framework.decorators import action
 from rest_framework.response import Response
@@ -15,11 +16,24 @@ class BaseHistoryModelMixin:
 class HistoryOnlyMixin(BaseHistoryModelMixin):
     @action(detail=True, methods=['GET'], name='Get History')
     def history(self, request, pk=None):
-        object = self.get_object()
-        versions = Version.objects.get_for_object(object)
+        instance = self.get_object()
+        versions = Version.objects.get_for_object(instance).order_by(
+            '-revision__date_created'
+        )
+        page = self.paginate_queryset(versions)
+        if page is not None:
+            serializer = self.version_serializer(page, many=True)
+            return self.get_paginated_response(serializer.data)
         serializer = self.version_serializer(versions, many=True)
         return Response(serializer.data)
 
+    @action(detail=True, methods=['GET'], name='Get Historic Version', url_path=r'history/(?P<version_pk>\d+)')
+    def version(self, request, pk=None, version_pk=None):
+        instance = self.get_object()
+        version = get_object_or_404(Version.objects.get_for_object(instance), id=version_pk)
+        serializer = self.version_serializer(version)
+        return Response(serializer.data)
+
 
 class DeletedOnlyMixin(BaseHistoryModelMixin):
     version_model = None
@@ -52,8 +66,8 @@ class RevertMixin(HistoryOnlyMixin):
                 {'error': 'Invalid Version Id'},
                 status=status.HTTP_400_BAD_REQUEST,
             )
-        object = self.get_object()
-        versions = Version.objects.get_for_object_reference(object, pk)
+        instance = self.get_object()
+        versions = Version.objects.get_for_object_reference(instance, pk)
         version = versions.filter(pk=version_pk).first()
         if not version:
             return Response(
@@ -62,9 +76,9 @@ class RevertMixin(HistoryOnlyMixin):
             )
         try:
             version.revision.revert()
-            object.refresh_from_db()
+            instance.refresh_from_db()
             with reversion.create_revision():
-                object.save()
+                instance.save()
                 reversion.set_user(request.user)
                 reversion.set_comment(
                     'Reverted to version {}'.format(version_pk))
